@@ -12,7 +12,7 @@ from typing_extensions import Self
 
 from ffmpeg import types
 from ffmpeg.asyncio.utils import create_subprocess, ensure_stream_reader, read_stream, readlines
-from ffmpeg.ffmpeg import FFmpegError
+from ffmpeg.ffmpeg import FFmpegAlreadyExecuted, FFmpegError
 from ffmpeg.options import Options
 from ffmpeg.progress import Tracker
 from ffmpeg.utils import is_windows
@@ -38,6 +38,15 @@ class FFmpeg(AsyncIOEventEmitter):
 
         self.once("error", self._reraise_exception)
 
+    @property
+    def arguments(self) -> list[str]:
+        """Return a list of arguments to be used when executing FFmpeg.
+
+        Returns:
+            A lit of arguments to be used when executing FFmpeg.
+        """
+        return [self._executable, *self._options.build()]
+
     def option(self, key: str, value: Optional[types.Option] = None) -> Self:
         """Add a global option `-key` or `-key value`.
 
@@ -57,7 +66,7 @@ class FFmpeg(AsyncIOEventEmitter):
         options: Optional[dict[str, Optional[types.Option]]] = None,
         **kwargs: Optional[types.Option],
     ) -> Self:
-        """Add an input file.
+        """Add an input file with specified options.
            By calling this method multiple times, an arbitrary number of input files can be added.
 
         Args:
@@ -65,6 +74,29 @@ class FFmpeg(AsyncIOEventEmitter):
             options: Options for the input file. Defaults to None.
             kwargs: Additional options for the input file.
 
+        Note:
+            Options for an input file can be specified in two ways:
+
+            Using `options`:
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4", {"codec:v": "libx264"}).output("output.mp4")
+            # Corresponds to `ffmpeg -codec:v libx264 -i input.mp4 output.mp4`
+            ```
+
+            Using `**kwargs`:
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4", vcodec="libx264").output("output.mp4")
+            # Corresponds to `ffmpeg -vcodec libx264 -i input.mp4 output.mp4`
+            ```
+
+        Note:
+            If an option does not require a value, use `None` for its value.
+
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4", ignore_unknown=None).output("output.mp4")
+            # Corresponds to `ffmpeg -ignore_unknown -i input.mp4 output.mp4`
+            ```
+
         Returns:
             An instance of `FFmpeg` itself, so that calls can be chained.
         """
@@ -77,7 +109,7 @@ class FFmpeg(AsyncIOEventEmitter):
         options: Optional[dict[str, Optional[types.Option]]] = None,
         **kwargs: Optional[types.Option],
     ) -> Self:
-        """Add an output file.
+        """Add an output file with specified options.
            By calling this method multiple times, an arbitrary number of output files can be specified.
 
         Args:
@@ -85,6 +117,29 @@ class FFmpeg(AsyncIOEventEmitter):
             options: Options for the output file. Defaults to None.
             kwargs: Additional options for the output file.
 
+        Note:
+            Options for an output file can be specified in two ways:
+
+            Using `options`:
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4").output("output.mp4", {"codec:v": "libx264"})
+            # Corresponds to `ffmpeg -i input.mp4 -codec:v libx264 output.mp4`
+            ```
+
+            Using `**kwargs`:
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4").output("output.mp4", vcodec="libx264")
+            # Corresponds to `ffmpeg -i input.mp4 -vcodec libx264 output.mp4`
+            ```
+
+        Note:
+            If an option does not require a value, use `None` for its value.
+
+            ```python
+            ffmpeg = FFmpeg().input("input.mp4").output("output.mp4", ignore_unknown=None)
+            # Corresponds to `ffmpeg -i input.mp4 -ignore_unknown output.mp4`
+            ```
+
         Returns:
             An instance of `FFmpeg` itself, so that calls can be chained.
         """
@@ -98,14 +153,14 @@ class FFmpeg(AsyncIOEventEmitter):
             stream: A stream to input to the standard input. Defaults to None.
 
         Raises:
-            FFmpegError: If FFmpeg is already executed.
+            FFmpegAlreadyExecuted: If FFmpeg is already executed.
             FFmpegError: If FFmpeg process returns non-zero exit status.
 
         Returns:
             The output to the standard output.
         """
         if self._executed:
-            raise FFmpegError("FFmpeg is already executed")
+            raise FFmpegAlreadyExecuted("FFmpeg is already executed", arguments=self.arguments)
 
         self._executed = False
         self._terminated = False
@@ -113,11 +168,10 @@ class FFmpeg(AsyncIOEventEmitter):
         if stream is not None:
             stream = ensure_stream_reader(stream)
 
-        arguments = [self._executable, *self._options.build()]
-        self.emit("start", arguments)
+        self.emit("start", self.arguments)
 
         self._process = await create_subprocess(
-            *arguments,
+            *self.arguments,
             stdin=subprocess.PIPE if stream is not None else None,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
@@ -125,7 +179,7 @@ class FFmpeg(AsyncIOEventEmitter):
 
         self._executed = True
 
-        tasks = [
+        tasks: list[asyncio.Task] = [
             asyncio.create_task(self._write_stdin(stream)),
             asyncio.create_task(self._read_stdout()),
             asyncio.create_task(self._handle_stderr()),
@@ -140,7 +194,7 @@ class FFmpeg(AsyncIOEventEmitter):
         elif self._terminated:
             self.emit("terminated")
         else:
-            raise FFmpegError(f"Non-zero exit status {self._process.returncode}")
+            raise FFmpegError.create(message=tasks[2].result(), arguments=self.arguments)
 
         return tasks[1].result()
 
@@ -151,7 +205,7 @@ class FFmpeg(AsyncIOEventEmitter):
             FFmpegError: If FFmpeg is not executed
         """
         if not self._executed:
-            raise FFmpegError("FFmpeg is not executed")
+            raise FFmpegError("FFmpeg is not executed", arguments=self.arguments)
 
         sigterm = signal.SIGTERM
         if is_windows():
@@ -187,11 +241,14 @@ class FFmpeg(AsyncIOEventEmitter):
 
         return bytes(buffer)
 
-    async def _handle_stderr(self):
+    async def _handle_stderr(self) -> str:
         assert self._process.stderr is not None
 
+        line = b""
         async for line in readlines(self._process.stderr):
             self.emit("stderr", line.decode())
 
+        return line.decode()
+
     def _reraise_exception(self, exception: Exception):
         raise exception
